@@ -28,6 +28,7 @@
 #include <glib-object.h>
 
 #include <gst/gstcontrolsource.h>
+#include <gst/controller/controller-prelude.h>
 
 G_BEGIN_DECLS
 
@@ -85,7 +86,12 @@ struct _GstDirectControlBinding {
   GstDirectControlBindingConvertValue convert_value;
   GstDirectControlBindingConvertGValue convert_g_value;
 
-  gpointer _gst_reserved[GST_PADDING];
+  union {
+    gpointer _gst_reserved[GST_PADDING];
+    struct {
+      gboolean want_absolute;
+    } abi;
+  } ABI;
 };
 
 /**
@@ -104,12 +110,22 @@ struct _GstDirectControlBindingClass
   gpointer _gst_reserved[GST_PADDING];
 };
 
+GST_CONTROLLER_API
 GType gst_direct_control_binding_get_type (void);
 
 /* Functions */
 
+GST_CONTROLLER_API
 GstControlBinding * gst_direct_control_binding_new (GstObject * object, const gchar * property_name,
                                                     GstControlSource * cs);
+GST_CONTROLLER_API
+GstControlBinding * gst_direct_control_binding_new_absolute (GstObject * object, const gchar * property_name, 
+                                                    GstControlSource * cs);
+
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDirectControlBinding, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_DIRECT_CONTROL_BINDING_H__ */
