@@ -25,6 +25,7 @@
  */
 /**
  * SECTION:gstnetclientclock
+ * @title: GstNetClientClock
  * @short_description: Special clock that synchronizes to a remote time
  *                     provider.
  * @see_also: #GstClock, #GstNetTimeProvider, #GstPipeline
@@ -60,6 +61,7 @@
 #include "gstnettimepacket.h"
 #include "gstntppacket.h"
 #include "gstnetclientclock.h"
+#include "gstnetutils.h"
 
 #include <gio/gio.h>
 
@@ -104,6 +106,7 @@ G_GNUC_INTERNAL GType gst_net_client_internal_clock_get_type (void);
  * more often than 1/20th second (arbitrarily, to spread observations a little) */
 #define DEFAULT_MINIMUM_UPDATE_INTERVAL (GST_SECOND / 20)
 #define DEFAULT_BASE_TIME       0
+#define DEFAULT_QOS_DSCP        -1
 
 /* Maximum number of clock updates we can skip before updating */
 #define MAX_SKIPPED_UPDATES 5
@@ -120,7 +123,8 @@ enum
   PROP_BUS,
   PROP_BASE_TIME,
   PROP_INTERNAL_CLOCK,
-  PROP_IS_NTP
+  PROP_IS_NTP,
+  PROP_QOS_DSCP
 };
 
 struct _GstNetClientInternalClock
@@ -146,6 +150,7 @@ struct _GstNetClientInternalClock
   gchar *address;
   gint port;
   gboolean is_ntp;
+  gint qos_dscp;
 
   /* Protected by OBJECT_LOCK */
   GList *busses;
@@ -210,6 +215,7 @@ gst_net_client_internal_clock_init (GstNetClientInternalClock * self)
   self->port = DEFAULT_PORT;
   self->address = g_strdup (DEFAULT_ADDRESS);
   self->is_ntp = FALSE;
+  self->qos_dscp = DEFAULT_QOS_DSCP;
 
   gst_clock_set_timeout (GST_CLOCK (self), DEFAULT_TIMEOUT);
 
@@ -634,6 +640,7 @@ gst_net_client_internal_clock_thread (gpointer data)
   GstNetClientInternalClock *self = data;
   GSocket *socket = self->socket;
   GError *err = NULL;
+  gint cur_qos_dscp = DEFAULT_QOS_DSCP;
 
   GST_INFO_OBJECT (self, "net client clock thread running, socket=%p", socket);
 
@@ -661,9 +668,19 @@ gst_net_client_internal_clock_thread (gpointer data)
         g_clear_error (&err);
         break;
       } else if (err->code == G_IO_ERROR_TIMED_OUT) {
+        gint new_qos_dscp;
+
         /* timed out, let's send another packet */
         GST_DEBUG_OBJECT (self, "timed out");
 
+        /* before next sending check if need to change QoS */
+        new_qos_dscp = self->qos_dscp;
+        if (cur_qos_dscp != new_qos_dscp &&
+            gst_net_utils_set_socket_dscp (socket, new_qos_dscp)) {
+          GST_DEBUG_OBJECT (self, "changed QoS DSCP to: %d", new_qos_dscp);
+          cur_qos_dscp = new_qos_dscp;
+        }
+
         if (self->is_ntp) {
           GstNtpPacket *packet;
 
@@ -964,6 +981,7 @@ struct _GstNetClientClockPrivate
 
   gchar *address;
   gint port;
+  gint qos_dscp;
 
   GstBus *bus;
 
@@ -1051,6 +1069,12 @@ gst_net_client_clock_class_init (GstNetClientClockClass * klass)
           "Internal clock that directly slaved to the remote clock",
           GST_TYPE_CLOCK, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_QOS_DSCP,
+      g_param_spec_int ("qos-dscp", "QoS diff srv code point",
+          "Quality of Service, differentiated services code point (-1 default)",
+          -1, 63, DEFAULT_QOS_DSCP,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   clock_class->get_internal_time = gst_net_client_clock_get_internal_time;
 }
 
@@ -1067,6 +1091,7 @@ gst_net_client_clock_init (GstNetClientClock * self)
 
   priv->port = DEFAULT_PORT;
   priv->address = g_strdup (DEFAULT_ADDRESS);
+  priv->qos_dscp = DEFAULT_QOS_DSCP;
 
   priv->roundtrip_limit = DEFAULT_ROUNDTRIP_LIMIT;
   priv->minimum_update_interval = DEFAULT_MINIMUM_UPDATE_INTERVAL;
@@ -1083,6 +1108,7 @@ update_clock_cache (ClockCache * cache)
 {
   GstClockTime roundtrip_limit = 0, minimum_update_interval = 0;
   GList *l, *busses = NULL;
+  gint qos_dscp = DEFAULT_QOS_DSCP;
 
   GST_OBJECT_LOCK (cache->clock);
   g_list_free_full (GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->busses,
@@ -1104,12 +1130,15 @@ update_clock_cache (ClockCache * cache)
     else
       minimum_update_interval =
           MIN (minimum_update_interval, clock->priv->minimum_update_interval);
+
+    qos_dscp = MAX (qos_dscp, clock->priv->qos_dscp);
   }
   GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->busses = busses;
   GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->roundtrip_limit =
       roundtrip_limit;
   GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->minimum_update_interval =
       minimum_update_interval;
+  GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->qos_dscp = qos_dscp;
 
   GST_OBJECT_UNLOCK (cache->clock);
 }
@@ -1227,6 +1256,12 @@ gst_net_client_clock_set_property (GObject * object, guint prop_id,
       gst_object_unref (clock);
       break;
     }
+    case PROP_QOS_DSCP:
+      GST_OBJECT_LOCK (self);
+      self->priv->qos_dscp = g_value_get_int (value);
+      GST_OBJECT_UNLOCK (self);
+      update = TRUE;
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1283,6 +1318,11 @@ gst_net_client_clock_get_property (GObject * object, guint prop_id,
     case PROP_INTERNAL_CLOCK:
       g_value_set_object (value, self->priv->internal_clock);
       break;
+    case PROP_QOS_DSCP:
+      GST_OBJECT_LOCK (self);
+      g_value_set_int (value, self->priv->qos_dscp);
+      GST_OBJECT_UNLOCK (self);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1331,6 +1371,7 @@ gst_net_client_clock_constructed (GObject * object)
         g_object_new (GST_TYPE_NET_CLIENT_INTERNAL_CLOCK, "address",
         self->priv->address, "port", self->priv->port, "is-ntp",
         self->priv->is_ntp, NULL);
+    gst_object_ref_sink (cache->clock);
     clocks = g_list_prepend (clocks, cache);
 
     /* Not actually leaked but is cached for a while before being disposed,
@@ -1378,10 +1419,10 @@ gst_net_client_clock_get_internal_time (GstClock * clock)
  * @base_time: initial time of the clock
  *
  * Create a new #GstNetClientInternalClock that will report the time
- * provided by the #GstNetTimeProvider on @remote_address and 
+ * provided by the #GstNetTimeProvider on @remote_address and
  * @remote_port.
  *
- * Returns: a new #GstClock that receives a time from the remote
+ * Returns: (transfer full): a new #GstClock that receives a time from the remote
  * clock.
  */
 GstClock *
@@ -1399,6 +1440,9 @@ gst_net_client_clock_new (const gchar * name, const gchar * remote_address,
       g_object_new (GST_TYPE_NET_CLIENT_CLOCK, "name", name, "address",
       remote_address, "port", remote_port, "base-time", base_time, NULL);
 
+  /* Clear floating flag */
+  gst_object_ref_sink (ret);
+
   return ret;
 }
 
@@ -1425,7 +1469,7 @@ gst_ntp_clock_init (GstNtpClock * self)
  * Create a new #GstNtpClock that will report the time provided by
  * the NTPv4 server on @remote_address and @remote_port.
  *
- * Returns: a new #GstClock that receives a time from the remote
+ * Returns: (transfer full): a new #GstClock that receives a time from the remote
  * clock.
  *
  * Since: 1.6
@@ -1445,5 +1489,7 @@ gst_ntp_clock_new (const gchar * name, const gchar * remote_address,
       g_object_new (GST_TYPE_NTP_CLOCK, "name", name, "address", remote_address,
       "port", remote_port, "base-time", base_time, NULL);
 
+  gst_object_ref_sink (ret);
+
   return ret;
 }
