@@ -24,6 +24,7 @@
 
 /**
  * SECTION:element-funnel
+ * @title: funnel
  *
  * Takes packets from various input sinks into one output source.
  *
@@ -127,7 +128,7 @@ static void
 gst_funnel_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstFunnel *funnel = GST_FUNNEL (object);
+  GstFunnel *funnel = GST_FUNNEL_CAST (object);
 
   switch (prop_id) {
     case PROP_FORWARD_STICKY_EVENTS:
@@ -143,7 +144,7 @@ static void
 gst_funnel_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
 {
-  GstFunnel *funnel = GST_FUNNEL (object);
+  GstFunnel *funnel = GST_FUNNEL_CAST (object);
 
   switch (prop_id) {
     case PROP_FORWARD_STICKY_EVENTS:
@@ -158,7 +159,7 @@ gst_funnel_get_property (GObject * object, guint prop_id, GValue * value,
 static void
 gst_funnel_dispose (GObject * object)
 {
-  GstFunnel *funnel = GST_FUNNEL (object);
+  GstFunnel *funnel = GST_FUNNEL_CAST (object);
   GList *item;
 
   gst_object_replace ((GstObject **) & funnel->last_sinkpad, NULL);
@@ -277,7 +278,7 @@ done:
 static void
 gst_funnel_release_pad (GstElement * element, GstPad * pad)
 {
-  GstFunnel *funnel = GST_FUNNEL (element);
+  GstFunnel *funnel = GST_FUNNEL_CAST (element);
   GstFunnelPad *fpad = GST_FUNNEL_PAD_CAST (pad);
   gboolean got_eos;
   gboolean send_eos = FALSE;
@@ -349,7 +350,7 @@ static GstFlowReturn
 gst_funnel_sink_chain_list (GstPad * pad, GstObject * parent,
     GstBufferList * list)
 {
-  GstFunnel *funnel = GST_FUNNEL (parent);
+  GstFunnel *funnel = GST_FUNNEL_CAST (parent);
 
   return gst_funnel_sink_chain_object (pad, funnel, TRUE,
       GST_MINI_OBJECT_CAST (list));
@@ -358,7 +359,7 @@ gst_funnel_sink_chain_list (GstPad * pad, GstObject * parent,
 static GstFlowReturn
 gst_funnel_sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 {
-  GstFunnel *funnel = GST_FUNNEL (parent);
+  GstFunnel *funnel = GST_FUNNEL_CAST (parent);
 
   return gst_funnel_sink_chain_object (pad, funnel, FALSE,
       GST_MINI_OBJECT_CAST (buffer));
@@ -367,7 +368,7 @@ gst_funnel_sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 static gboolean
 gst_funnel_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  GstFunnel *funnel = GST_FUNNEL (parent);
+  GstFunnel *funnel = GST_FUNNEL_CAST (parent);
   GstFunnelPad *fpad = GST_FUNNEL_PAD_CAST (pad);
   gboolean forward = TRUE;
   gboolean res = TRUE;
@@ -397,15 +398,23 @@ gst_funnel_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     GST_OBJECT_LOCK (funnel);
     fpad->got_eos = FALSE;
     GST_OBJECT_UNLOCK (funnel);
-  } else if (GST_EVENT_TYPE (event) == GST_EVENT_GAP) {
-    /* If no data is coming and we receive GAP event, need to forward sticky events. */
-    unlock = TRUE;
-    GST_PAD_STREAM_LOCK (funnel->srcpad);
-    GST_OBJECT_LOCK (funnel);
-    gst_object_replace ((GstObject **) & funnel->last_sinkpad,
-        GST_OBJECT (pad));
-    GST_OBJECT_UNLOCK (funnel);
-    gst_pad_sticky_events_foreach (pad, forward_events, funnel->srcpad);
+  }
+
+  if (forward && GST_EVENT_IS_SERIALIZED (event)) {
+    /* If no data is coming and we receive serialized event, need to forward all sticky events.
+     * Otherwise downstream has an inconsistent set of sticky events when
+     * handling the new event. */
+    if (!unlock) {
+      unlock = TRUE;
+      GST_PAD_STREAM_LOCK (funnel->srcpad);
+    }
+
+    if ((funnel->last_sinkpad == NULL) || (funnel->forward_sticky_events
+            && (funnel->last_sinkpad != pad))) {
+      gst_object_replace ((GstObject **) & funnel->last_sinkpad,
+          GST_OBJECT (pad));
+      gst_pad_sticky_events_foreach (pad, forward_events, funnel->srcpad);
+    }
   }
 
   if (forward)
