@@ -17,6 +17,13 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#ifndef GST_REMOVE_DEPRECATED
+#undef GST_DISABLE_DEPRECATED
+#endif
 
 #include <gst/check/gstcheck.h>
 
@@ -75,6 +82,39 @@ GST_START_TEST (test_uri_get_location)
 
 GST_END_TEST;
 
+#ifndef GST_REMOVE_DEPRECATED
+GST_START_TEST (test_gst_uri_construct)
+{
+  gchar *l;
+
+  /* URI with no protocol or empty protocol should return empty string */
+  ASSERT_CRITICAL (l = gst_uri_construct (NULL, "/path/to/file"));
+  fail_unless (l == NULL);
+  ASSERT_CRITICAL (l = gst_uri_construct ("", "/path/to/file"));
+  fail_unless (l == NULL);
+
+  /* URI with no location should return empty string */
+  ASSERT_CRITICAL (l = gst_uri_construct ("protocol", NULL));
+  fail_unless (l == NULL);
+
+  /* check the protocol for validity */
+  l = gst_uri_construct ("protocol1234567890+-.", "somefile");
+  fail_unless (l != NULL);
+  fail_unless_equals_string (l, "protocol1234567890+-.://somefile");
+  g_free (l);
+
+  /* check the location for correct handling */
+  l = gst_uri_construct ("aprotocol",
+      "/path+ to/some/file%d?akey=aval&key2=val2");
+  fail_unless (l != NULL);
+  fail_unless_equals_string (l,
+      "aprotocol:///path%2B%20to/some/file%25d?akey=aval&key2=val2");
+  g_free (l);
+}
+
+GST_END_TEST;
+#endif
+
 #ifdef G_OS_WIN32
 
 GST_START_TEST (test_win32_uri)
@@ -980,6 +1020,80 @@ GST_START_TEST (test_url_get_set)
 
 GST_END_TEST;
 
+GST_START_TEST (test_url_get_media_fragment_table)
+{
+  GstUri *url;
+  gchar *val;
+  GHashTable *table;
+
+  /* Examples at https://www.w3.org/TR/media-frags/#processing-media-fragment-uri */
+
+  /* TEST "t=1" */
+  url = gst_uri_from_string ("http://foo/var/file#t=1");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "t", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("1", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* NOTE: Media Fragments URI 1.0 (W3C) is saying that
+   * "Multiple occurrences of the same dimension: only the last valid occurrence
+   *  of a dimension (e.g. t=10 in #t=2&t=10) is interpreted and all previous
+   *  occurrences (valid or invalid) SHOULD be ignored by the user agent"
+   */
+  /* TEST "t=1&t=2" */
+  url = gst_uri_from_string ("http://foo/var/file#t=1&t=2");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "t", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("2", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* TEST "a=b=c" */
+  url = gst_uri_from_string ("http://foo/var/file#a=b=c");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "a", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("b=c", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* TEST "a&b=c" */
+  url = gst_uri_from_string ("http://foo/var/file#a&b=c");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 2);
+  fail_unless (g_hash_table_lookup_extended (table, "a", NULL,
+          (gpointer) & val));
+  fail_unless (val == NULL);
+  fail_unless (g_hash_table_lookup_extended (table, "b", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("c", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* TEST "%74=%6ept%3A%310" */
+  url = gst_uri_from_string ("http://foo/var/file#%74=%6ept%3A%310");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "t", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("npt:10", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_uri_suite (void)
 {
@@ -991,6 +1105,9 @@ gst_uri_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_protocol_case);
   tcase_add_test (tc_chain, test_uri_get_location);
+#ifndef GST_REMOVE_DEPRECATED
+  tcase_add_test (tc_chain, test_gst_uri_construct);
+#endif
   tcase_add_test (tc_chain, test_uri_misc);
   tcase_add_test (tc_chain, test_element_make_from_uri);
 #ifdef G_OS_WIN32
@@ -1003,6 +1120,7 @@ gst_uri_suite (void)
   tcase_add_test (tc_chain, test_url_equality);
   tcase_add_test (tc_chain, test_url_constructors);
   tcase_add_test (tc_chain, test_url_get_set);
+  tcase_add_test (tc_chain, test_url_get_media_fragment_table);
 
   return s;
 }
